"""
Copyright (c) 2025-now Martian Bugs All rights reserved.
Build Date: 2025-02-18
Author: Martian Bugs
Description: 数据采集器
"""

from DrissionPage import Chromium, ChromiumOptions

from ._login import Login
from .ss.ss import Ss


class Collector:
    """采集器. 使用之前请先调用 `connect_browser` 方法连接浏览器."""

    def __init__(self):
        self._ss = None

    def connect_browser(self, port: int):
        """
        连接浏览器

        Args:
            port: 浏览器调试端口号
        """

        chrome_options = ChromiumOptions(read_file=False)
        chrome_options.set_local_port(port=port)

        self.browser = Chromium(addr_or_opts=chrome_options)

    def login(
        self,
        account: str,
        password: str,
    ):
        """
        聚水潭后台登录

        Args:
            account: 登录账号
            password: 登录密码
        Returns:
            如果登录成功, 将返回操作的浏览器标签页对象
        """

        login_utils = Login(browser=self.browser)
        return login_utils.login(account=account, password=password)

    @property
    def ss(self):
        """胜算模块数据采集"""

        if self._ss is None:
            self._ss = Ss(self.browser)

        return self._ss
